package org.syncloud.facade;

import syncloud.storage.Account;
import org.syncloud.facade.dao.AccountsDao;
import syncloud.core.CoreException;
import syncloud.core.log.Logger;
import syncloud.storage.ErrorCode;
import syncloud.storage.ISecured;
import syncloud.storage.IStorage;
import syncloud.storage.StorageKey;

public class AuthenticationManager {
    private static Logger logger = Logger.getLogger(AuthenticationManager.class);

    private ConfigurationManager configuration;
    private StorageManager storageManager;
    private AccountsDao accountsDao;

    public AuthenticationManager(ConfigurationManager configuration, StorageManager storageManager, AccountsDao accountsDao) {
        this.configuration = configuration;
        this.storageManager = storageManager;
        this.accountsDao = accountsDao;
    }

    public void addAccount(StorageKey storageKey, String password) {
        accountsDao.addAccount(new Account(storageKey, password));
    }

    public boolean authenticate(StorageKey storageKey, String password) {
        boolean authenticated = false;
        IStorage storage = storageManager.getStorage(storageKey);
        if (storage instanceof ISecured) {
            authenticated = ((ISecured) storage).authenticate(password);
            if (authenticated)
                addAccount(storageKey, password);
        }
        return authenticated;
    }

    public boolean isAuthenticated(StorageKey storageKey) {
        IStorage storage = storageManager.getStorage(storageKey);
        if (storage instanceof ISecured)
            return ((ISecured) storage).authenticated();
        return true;
    }

    public boolean authenticated(StorageKey storageKey) {
        return isAuthenticated(storageKey) || authenticateSaved(storageKey);
    }

    private boolean authenticateSaved(StorageKey storageKey) {
        IStorage storage = storageManager.getStorage(storageKey);
        ISecured securedStorage = (ISecured) storage;
        Account account = accountsDao.getUserAccount(storageKey);
        if (account != null) {
            try {
                return securedStorage.authenticate(account.getPassword());
            } catch (Exception e) {
                throw new CoreException(ErrorCode.AuthenticationFailed, e.getMessage());
            }
        }
        return false;
    }
}