package org.syncloud.facade.dao;

import syncloud.storage.Account;
import syncloud.storage.StorageKey;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class AccountsDao {
    private ConfigReader<List<Account>> configReader;
    private ConfigWriter<List<Account>> configWriter;

    public AccountsDao(ConfigReader<List<Account>> configReader, ConfigWriter<List<Account>> configWriter) {
        this.configReader = configReader;
        this.configWriter = configWriter;
    }

    public List<Account> getAccounts(String storageId) {
        List<Account> accounts = new ArrayList<Account>();
        for (Account account : getUserAccounts()) {
            if (account.getStorageKey().getStorageId().equals(storageId))
                accounts.add(account);
        }
        return accounts;
    }

    public List<Account> getUserAccounts() {
        List<Account> accounts = configReader.read();
        if (accounts == null)
            accounts = new ArrayList<Account>();
        else
            accounts = new ArrayList<Account>(accounts);

        return accounts;
    }

    public Account getUserAccount(StorageKey storageKey) {
        List<Account> userAccounts = getUserAccounts();
        for (Account account : userAccounts) {
            if (account.getStorageKey().getUser().equals(storageKey.getUser()))
                return account;
        }
        return null;
    }

    public void addAccount(Account newAccount) {

        Account foundAccount = null;
        List<Account> accounts = getUserAccounts();
        for (Account account : accounts) {
            if (account.equals(newAccount))
                foundAccount = account;
        }

        if (foundAccount == null)
            accounts.add(newAccount);
        else {
            foundAccount.setPassword(newAccount.getPassword());
        }

        configWriter.save(accounts);
    }

    public void deleteAccount(StorageKey storageKey) {

        List<Account> accounts = getUserAccounts();
        Iterator<Account> iterator = accounts.iterator();
        while (iterator.hasNext()) {
            Account account = iterator.next();
            if (account.getStorageKey().equals(storageKey)) {
                iterator.remove();
            }
        }

        configWriter.save(accounts);
    }

    public void deleteAllAccounts() {
        configWriter.save(new ArrayList<Account>());
    }

}
