package org.syncloud.facade.dao.json;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.syncloud.facade.dao.json.base.Parser;
import syncloud.storage.Account;
import syncloud.storage.StorageKey;
import syncloud.storage.User;

import java.util.LinkedList;
import java.util.List;

public class AccountParser implements Parser<List<Account>, String> {

    public static final String JSON_ROOT = "accounts";
    public static final String JSON_STORAGE_ID = "storageId";
    public static final String JSON_LOGIN = "login";
    public static final String JSON_PASSWORD = "password";

    @Override
    public String toJson(List<Account> accounts) throws JSONException {
        JSONObject jsonConf = new JSONObject();
        JSONArray jsonAccounts = new JSONArray();
        for (Account account : accounts) {
            jsonAccounts.put(toJson(account));
        }
        jsonConf.put(JSON_ROOT, jsonAccounts);
        return jsonConf.toString();

    }

    private JSONObject toJson(Account account) throws JSONException {
        JSONObject jsonAccount = new JSONObject();
        jsonAccount.put(JSON_STORAGE_ID, account.getStorageKey().getStorageId());
        jsonAccount.put(JSON_LOGIN, account.getUser().getLogin());
        jsonAccount.put(JSON_PASSWORD, account.getPassword());
        return jsonAccount;
    }

    @Override
    public List<Account> fromJson(String json) throws JSONException {

        JSONObject jsonObject = new JSONObject(json);
        List<Account> accounts = new LinkedList<Account>();
        JSONArray jsonAccounts = (JSONArray) jsonObject.get(JSON_ROOT);
        for (int i = 0; i < jsonAccounts.length(); i++) {
            JSONObject jsonAccount = jsonAccounts.getJSONObject(i);
            User user = new User(jsonAccount.getString(JSON_LOGIN));
            StorageKey storageKey = new StorageKey(jsonAccount.getString(JSON_STORAGE_ID), user);
            Account account = new Account(storageKey, jsonAccount.getString(JSON_PASSWORD));
            accounts.add(account);
        }
        return accounts;
    }
}
