package org.syncloud.facade.dao.json;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.syncloud.facade.dao.json.base.Reader;
import syncloud.core.CoreException;
import syncloud.core.log.Logger;
import syncloud.storage.IStorageMetadata;

import java.util.ArrayList;
import java.util.List;

public class StorageMetadataReader implements Reader<String, List<IStorageMetadata>> {

    private static Logger logger = Logger.getLogger(StorageMetadataReader.class);

    public static final String ROOT = "storages";
    private Class<IStorageMetadata> metadataClass = IStorageMetadata.class;

    @Override
    public List<IStorageMetadata> fromJson(String json) throws JSONException, CoreException {
        JSONObject jsonObject = new JSONObject(json);
        List<IStorageMetadata> metadataList = new ArrayList<IStorageMetadata>();
        JSONArray storages = jsonObject.getJSONArray(ROOT);
        for (int i = 0; i < storages.length(); i++) {
            String className = storages.getString(i);
            IStorageMetadata meta = create(className);
            if (meta != null)
                metadataList.add(meta);
        }
        return metadataList;
    }

    private IStorageMetadata create(String className) {
        try {
            Class cls = Class.forName(className);
            if ((metadataClass.isAssignableFrom(cls))) {
                return (IStorageMetadata) cls.newInstance();
            }
            logger.error("metadata class: " + className + " is not derived from " + metadataClass.getCanonicalName() + ", skipping ...");
        } catch (ClassNotFoundException e) {
            logger.error("metadata class: " + className + " not found ", e);
        } catch (InstantiationException e) {
            logger.error("metadata class: " + className + " unable to initialize ", e);
        } catch (IllegalAccessException e) {
            logger.error("metadata class: " + className + " unable to access ", e);
        }
        return null;
    }
}
