package org.syncloud.facade.dao.json;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.syncloud.facade.dao.json.base.Parser;
import syncloud.core.CoreException;
import syncloud.storage.Sync;
import syncloud.storage.NodeKey;
import syncloud.storage.StorageKey;
import syncloud.storage.User;

import java.util.LinkedList;
import java.util.List;

public class SyncParser implements Parser<List<Sync>, String> {

    public static final String JSON_SYNCS = "synchronizations";
    public static final String JSON_SYNC_NAME = "name";
    public static final String JSON_SYNC_PATHS = "paths";
    public static final String JSON_SYNC_PATH_STORAGE_ID = "storageId";
    public static final String JSON_SYNC_PATH_LOGIN = "login";
    public static final String JSON_SYNC_PATH_PATH = "path";

    @Override
    public String toJson(List<Sync> syncs) throws JSONException {
        JSONObject jsonConf = new JSONObject();
        JSONArray jsonSyncs = new JSONArray();
        for (Sync sync : syncs) {
            JSONObject jsonSync = new JSONObject();
            jsonSync.put(JSON_SYNC_NAME, sync.getName());
            JSONArray jsonPaths = toPathsJson(sync.getPaths());
            jsonSync.put(JSON_SYNC_PATHS, jsonPaths);
            jsonSyncs.put(jsonSync);
        }
        jsonConf.put(JSON_SYNCS, jsonSyncs);
        return jsonConf.toString();
    }

    private JSONArray toPathsJson(List<NodeKey> paths) throws JSONException {
        JSONArray jsonPaths = new JSONArray();
        for (NodeKey path : paths) {
            JSONObject jsonPath = new JSONObject();
            jsonPath.put(JSON_SYNC_PATH_STORAGE_ID, path.getStorageKey().getStorageId());
            jsonPath.put(JSON_SYNC_PATH_LOGIN, path.getStorageKey().getUser().getLogin());
            jsonPath.put(JSON_SYNC_PATH_PATH, path.getNativePath());
            jsonPaths.put(jsonPath);
        }
        return jsonPaths;
    }

    @Override
    public List<Sync> fromJson(String jsonConf) throws JSONException, CoreException {

        List<Sync> syncs = new LinkedList<Sync>();

        if (jsonConf.trim().isEmpty())
            return syncs;

        JSONObject json = new JSONObject(jsonConf);
        JSONArray jsonSyncs = (JSONArray) json.get(JSON_SYNCS);
        for (int i = 0; i < jsonSyncs.length(); i++) {
            JSONObject jsonSync = jsonSyncs.getJSONObject(i);
            List<NodeKey> nodeKeys = new LinkedList<NodeKey>();
            JSONArray jsonPaths = jsonSync.getJSONArray(JSON_SYNC_PATHS);
            for (int j = 0; j < jsonPaths.length(); j++) {
                JSONObject jsonPath = jsonPaths.getJSONObject(j);
                NodeKey nodeKey = NodeKey.create(
                        new StorageKey(
                                jsonPath.getString(JSON_SYNC_PATH_STORAGE_ID),
                                new User(jsonPath.getString(JSON_SYNC_PATH_LOGIN))
                        ),
                        jsonPath.getString(JSON_SYNC_PATH_PATH));
                nodeKeys.add(nodeKey);
            }
            Sync sync = new Sync(jsonSync.getString(JSON_SYNC_NAME), nodeKeys);
            syncs.add(sync);
        }
        return syncs;
    }
}